package httpserver

import (
	"fmt"
	benefactorhandler "git.gocasts.ir/ebhomengo/niki/delivery/http_server/benefactor/benefactor"
	benefactorservice "git.gocasts.ir/ebhomengo/niki/service/benefactor/benefactor"
	benefactorvalidator "git.gocasts.ir/ebhomengo/niki/validator/benefactor/benefactor"

	config "git.gocasts.ir/ebhomengo/niki/config"
	echo "github.com/labstack/echo/v4"
	middleware "github.com/labstack/echo/v4/middleware"
)

type Server struct {
	config            config.Config
	Router            *echo.Echo
	benefactorHandler benefactorhandler.Handler
}

func New(cfg config.Config, benefactorSvc benefactorservice.Service, benefactorVld benefactorvalidator.Validator) Server {
	return Server{
		Router:            echo.New(),
		config:            cfg,
		benefactorHandler: benefactorhandler.New(cfg.Auth, benefactorSvc, benefactorVld),
	}
}

func (s Server) Serve() {
	s.Router.Use(middleware.RequestID())

	s.Router.Use(middleware.Recover())

	// Routes
	s.Router.GET("/health-check", s.healthCheck)
	s.benefactorHandler.SetRoutes(s.Router)

	// Start server
	address := fmt.Sprintf(":%d", s.config.HTTPServer.Port)
	fmt.Printf("start echo server on %s\n", address)
	if err := s.Router.Start(address); err != nil {
		fmt.Println("router start error", err)
	}
}
